package com.igaworks.adbrixrm_flutter;

import android.util.Log;

import com.igaworks.v2.core.AdBrixRm;

import android.os.Handler;
import android.os.Looper;

import org.json.JSONException;
import org.json.JSONObject;

import io.adbrix.sdk.component.AbxLog;
import io.adbrix.sdk.domain.model.Empty;
import io.adbrix.sdk.domain.model.Result;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;

class AdbrixrmEventChannel implements EventChannel.StreamHandler, AdBrixRm.DeeplinkListener,
        AdBrixRm.DeferredDeeplinkListener, AdBrixRm.InAppMessageClickListener, 
        AdBrixRm.onTouchRemotePushListener {
    private static final String ARGUMENT_Deeplink = "com.adbrix.stream.deeplink";
    private static final String ARGUMENT_DeferredDeeplink = "com.adbrix.stream.deferredDeeplink";
    private static final String ARGUMENT_PushRemoteCallback = "com.adbrix.stream.pushRemoteCallback";
    private static final String ARGUMENT_OnReceiveInAppMessageClick = "com.adbrix.stream.onReceiveInAppMessageClick";

    private static final String EVENT_CHANNEL_Deeplink = "com.adbrix.eventChannel.deeplink";
    private static final String EVENT_CHANNEL_DeferredDeeplink = "com.adbrix.eventChannel.deferredDeeplink";
    private static final String EVENT_CHANNEL_PushRemoteCallback = "com.adbrix.eventChannel.pushRemoteCallback";
    private static final String EVENT_CHANNEL_OnReceiveInAppMessageClick = "com.adbrix.eventChannel.onReceiveInAppMessageClick";

    private EventChannel eventChannelDeeplink;
    private EventChannel eventChannelDeferredDeeplink;
    private EventChannel eventChannelPushRemoteCallback;
    private EventChannel eventChannelOnReceiveInAppMessageClick;

    public static final String ERROR_CODE_SDK_ERROR = "-1";

    private EventChannel.EventSink sinkDeeplink;
    private EventChannel.EventSink sinkDeferredDeeplink;
    private EventChannel.EventSink sinkPushRemote;
    private EventChannel.EventSink sinkOnReceiveInAppMessage;

    private String initialDeeplink;
    private String initialPushData;

    private static final AdbrixrmEventChannel instance = new AdbrixrmEventChannel();

    private AdbrixrmEventChannel(){}

    public static AdbrixrmEventChannel shared() {
        return instance;
    }

    public void linkEventChannel(BinaryMessenger messenger) {
        this.eventChannelDeeplink = new EventChannel(messenger, EVENT_CHANNEL_Deeplink);
        this.eventChannelDeferredDeeplink = new EventChannel(messenger, EVENT_CHANNEL_DeferredDeeplink);
        this.eventChannelPushRemoteCallback = new EventChannel(messenger, EVENT_CHANNEL_PushRemoteCallback);
        this.eventChannelOnReceiveInAppMessageClick = new EventChannel(messenger, EVENT_CHANNEL_OnReceiveInAppMessageClick);
    }

    void setStreamHandler() {
        eventChannelDeeplink.setStreamHandler(this);
        eventChannelDeferredDeeplink.setStreamHandler(this);
        eventChannelPushRemoteCallback.setStreamHandler(this);
        eventChannelOnReceiveInAppMessageClick.setStreamHandler(this);
    }

    void unSetStreamHandler() {
        eventChannelDeeplink.setStreamHandler(null);
        eventChannelDeferredDeeplink.setStreamHandler(null);
        eventChannelPushRemoteCallback.setStreamHandler(null);
        eventChannelOnReceiveInAppMessageClick.setStreamHandler(null);

        this.eventChannelDeeplink = null;
        this.eventChannelDeferredDeeplink = null;
        this.eventChannelPushRemoteCallback = null;
        this.eventChannelOnReceiveInAppMessageClick = null;
        
        onCancel("");
    }

    @Override
    public void onListen(Object arguments, final EventChannel.EventSink events) {
        String argument = arguments.toString();
        switch (argument) {
            case ARGUMENT_Deeplink:
                this.sinkDeeplink = events;
                if (initialDeeplink != null) {
                    if (sinkDeeplink != null) {
                        String _initialDeeplink = initialDeeplink;
                        sinkDeeplink.success(_initialDeeplink);
                        initialDeeplink = null;
                    }
                }
                break;
            case ARGUMENT_DeferredDeeplink:
                this.sinkDeferredDeeplink = events;
                break;
            case ARGUMENT_PushRemoteCallback:
                this.sinkPushRemote = events;
                if (initialPushData != null) {
                    if (sinkPushRemote != null) {
                        String _initialPushData = initialPushData;
                        sinkPushRemote.success(_initialPushData);
                        initialPushData = null;
                    }
                }
                break;
            case ARGUMENT_OnReceiveInAppMessageClick:
                this.sinkOnReceiveInAppMessage = events;
                break;
        }

    }

    @Override
    public void onCancel(Object arguments) {
        this.sinkDeeplink = null;
        this.sinkDeferredDeeplink = null;
        this.sinkPushRemote = null;
        this.sinkOnReceiveInAppMessage = null;
    }

    @Override
    public void onReceiveDeeplink(String s) {
        final String deeplink = s;
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (sinkDeeplink != null) {
                    sinkDeeplink.success(deeplink);
                } else {
                    initialDeeplink = deeplink;
                }
            }
        });
    }

    @Override
    public void onReceiveDeferredDeeplink(String s) {
        final String deferredDeeplink = s;
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (sinkDeferredDeeplink != null) {
                    sinkDeferredDeeplink.success(deferredDeeplink);
                }
            }
        });
    }

    @Override
    public void onReceiveInAppMessageClick(String s, String s1, String s2, boolean b) {

        final JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("actionId", s);
            jsonObject.put("actionType", s1);
            jsonObject.put("actionArg", s2);
            jsonObject.put("isClosed", b);
        } catch (JSONException e) {
            AbxLog.e("onReceiveInAppMessageClick() parse error", e, false);
        }
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (sinkOnReceiveInAppMessage != null) {
                    sinkOnReceiveInAppMessage.success(jsonObject.toString());
                }
                if (sinkDeeplink != null) {
                    if (s1.equals("deeplink") || s1.equals("deeplink_and_close")) {
                        sinkDeeplink.success(s2);
                    }
                }
            }
        });
    }

    @Override
    public void onTouchRemotePush(String s) {
        final String pushData = s;
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (sinkPushRemote != null) {
                    sinkPushRemote.success(pushData);
                } else {
                    initialPushData = pushData;
                }
            }
        });
    }
}
