package com.igaworks.adbrixrm_flutter;

import android.app.Activity;
import android.app.Application;
import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.core.app.NotificationCompat;

import com.igaworks.v2.core.AbxCommerce;
import com.igaworks.v2.core.AbxCommon;
import com.igaworks.v2.core.AbxGame;
import com.igaworks.v2.core.AdBrixRm;
import com.igaworks.v2.core.application.AbxActivityHelper;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.adbrix.sdk.component.AbxLog;
import io.adbrix.sdk.domain.function.Completion;
import io.adbrix.sdk.domain.model.ActionHistory;
import io.adbrix.sdk.domain.model.DfnInAppMessage;
import io.adbrix.sdk.domain.model.DfnInAppMessageFetchMode;
import io.adbrix.sdk.domain.model.Empty;
import io.adbrix.sdk.utils.CommonUtils;
import io.adbrix.sdk.domain.model.SubscriptionStatus;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.google.firebase.messaging.RemoteMessage;
import com.igaworks.v2.core.result.GetSubscriptionStatusResult;
import com.igaworks.v2.core.result.SetCiProfileResult;
import com.igaworks.v2.core.result.SetSubscriptionStatusResult;

/**
 * AdbrixrmflutterPlugin
 */
public class AdbrixrmFlutterPlugin implements FlutterPlugin, ActivityAware, MethodCallHandler {
    private static final String METHOD_CHANNEL = "com.adbrix.methodChannel";

    public static final String ERROR_CODE_SDK_ERROR = "-1";
    private Activity activity;
    private Context context;
    private MethodChannel methodChannel;

    public static void sdkInit(Application application, String appkey, String secretKey) {
        AbxActivityHelper.initializeSdk(application, appkey, secretKey);
    }

    public static void setRegistrationId(String token) {
        AdBrixRm.setRegistrationId(token);
    }

    public static void setPushEnable(boolean enable) {
        AdBrixRm.setPushEnable(enable);
    }

    public static void setPushIconStyle(Context context, String smallIconName, String largeIconName, int argb) {
        AdBrixRm.setPushIconStyle(context, smallIconName, largeIconName, argb);
    }

    public static void setNotificationChannel(String channelName, String channelDescription) {
        AdBrixRm.createDefaultNotificationChannel(channelName, channelDescription);
    }

    public static void setNotificationOption(Context context, int priority, int visibility) {
        AdBrixRm.setNotificationOption(context, priority, visibility);
    }

    public static void deeplinkEventWithIntent(Intent intent) {
        AdBrixRm.deeplinkEventWithIntent(intent);
    }

    public static void setDeeplinkListener() {
        AdBrixRm.setDeeplinkListener(AdbrixrmEventChannel.shared());
    }

    public static void setDeferredDeeplinkListener() {
        AdBrixRm.setDeferredDeeplinkListener(AdbrixrmEventChannel.shared());
    }

    public static void setRemotePushMessageListener() {
        AdBrixRm.setRemotePushMessageListener(AdbrixrmEventChannel.shared());
    }

    public static void setInAppMessageClickListener() {
        AdBrixRm.setInAppMessageClickListener(AdbrixrmEventChannel.shared());
    }

    public static void onMessageReceived(Context context, RemoteMessage remoteMessage) {
        AdBrixRm.onMessageReceived(context, remoteMessage);
    }

    // region Flutter Event
    private void initPlugin(Context context, BinaryMessenger messenger) {
        this.context = context;
        this.methodChannel = new MethodChannel(messenger, METHOD_CHANNEL);
        this.methodChannel.setMethodCallHandler(this);

        AdbrixrmEventChannel.shared().linkEventChannel(messenger);
        AdbrixrmEventChannel.shared().setStreamHandler();
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        initPlugin(flutterPluginBinding.getApplicationContext(), flutterPluginBinding.getBinaryMessenger());
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        methodChannel.setMethodCallHandler(null);
        AdbrixrmEventChannel.shared().unSetStreamHandler();
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        onDetachedFromActivity();
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        onAttachedToActivity(binding);
    }

    @Override
    public void onDetachedFromActivity() {
        this.activity = null;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        String method = call.method;
        switch (method) {
            case "setEventUploadCountInterval":
                setEventUploadCountInterval(call);
                break;
            case "setEventUploadTimeInterval":
                setEventUploadTimeInterval(call);
                break;
            case "login":
                login(call);
                break;
            case "logout":
                logout();
                break;
            case "setAge":
                setAge(call);
                break;
            case "setGender":
                setGender(call);
                break;
            case "commonSignUp":
                commonSignUp(call);
                break;
            case "appUpdate":
                commonAppUpdate(call);
                break;
            case "userInvite":
                commonInvite(call);
                break;
            case "useCredit":
                commonUseCredit(call);
                break;
            case "commonPurchase":
                commonPurchase(call);
                break;
            case "event":
                events(call);
                break;
            case "commerceViewHome":
                commerceViewHome(call);
                break;
            case "commerceCategoryView":
                commerceCategoryView(call);
                break;
            case "commerceProductView":
                commerceProductView(call);
                break;
            case "commerceAddToCart":
                commerceAddToCart(call);
                break;
            case "commerceAddToWishList":
                commerceAddToWishList(call);
                break;
            case "commerceReviewOrder":
                commerceReviewOrder(call);
                break;
            case "commerceRefund":
                commerceRefund(call);
                break;
            case "commerceSearch":
                commerceSearch(call);
                break;
            case "commerceShare":
                commerceShare(call);
                break;
            case "commerceListView":
                commerceListView(call);
                break;
            case "commerceCartView":
                commerceCartView(call);
                break;
            case "commercePaymentInfoAdd":
                commercePaymentInfoAdd(call);
                break;
            case "gameTutorialComplete":
                gameTutorialComplete(call);
                break;
            case "gameCharacterCreated":
                gameCharacterCreated(call);
                break;
            case "gameStageCleared":
                gameStageCleared(call);
                break;
            case "gameLevelAchieved":
                gameLevelAchieved(call);
                break;
            case "gdprForgetMe":
                gdprForgetMe();
                break;
            case "setRegistrationId":
                setRegistrationId(call);
                break;
            case "setNotificationChannel":
                setNotificationChannel(call);
                break;
            case "setPushEnable":
                setPushEnable(call);
                break;
            case "fetchInAppMessage":
                fetchInAppMessage(result);
                break;
            case "getAllInAppMessage":
                getAllInAppMessage(result);
                break;
            case "openInAppMessage":
                openInAppMessage(call, result);
                break;
            case "setSubscriptionStatus":
                setSubscriptionStatus(call, result);
                break;
            case "getSubscriptionStatus":
                getSubscriptionStatus(result);
                break;
            case "setKakaoId":
                setKakoId(call, result);
                break;
            case "setPhoneNumber":
                setPhoneNumber(call, result);
                break;
        }
    }
    // endregion
    // region AdBrixRm

    private void login(MethodCall call) {
        String userId = call.arguments();
        AdBrixRm.login(userId);
    }

    private void logout() {
        AdBrixRm.logout();
    }

    private void gdprForgetMe() {
        AdBrixRm.gdprForgetMe(context);
    }

    private void setAge(MethodCall call) {
        int age = call.arguments();
        AdBrixRm.setAge(age);
    }

    private void setGender(MethodCall call) {
        String gender = call.arguments();
        AdBrixRm.setGender(AdBrixRm.AbxGender.valueOf(gender));
    }

    private void events(MethodCall call) {
        String eventName = call.argument("eventName");
        if (call.hasArgument("attr")) {
            HashMap<String, Object> attr = call.argument("attr");
            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.event(eventName, eventAttr);
                return;
            }
        }
        AdBrixRm.event(eventName);
    }

    private void setEventUploadCountInterval(MethodCall call) {
        String argument = call.arguments();
        AdBrixRm.AdBrixEventUploadCountInterval interval;
        switch (argument) {
            case "MIN":
                interval = AdBrixRm.AdBrixEventUploadCountInterval.MIN;
                break;
            case "NORMAL":
                interval = AdBrixRm.AdBrixEventUploadCountInterval.NORMAL;
                break;
            case "MAX":
                interval = AdBrixRm.AdBrixEventUploadCountInterval.MAX;
                break;
            default:
                interval = AdBrixRm.AdBrixEventUploadCountInterval.NORMAL;
                break;
        }
        AdBrixRm.setEventUploadCountInterval(interval);
    }

    private void setEventUploadTimeInterval(MethodCall call) {
        String argument = call.arguments();
        AdBrixRm.AdBrixEventUploadTimeInterval interval;
        switch (argument) {
            case "MIN":
                interval = AdBrixRm.AdBrixEventUploadTimeInterval.MIN;
                break;
            case "NORMAL":
                interval = AdBrixRm.AdBrixEventUploadTimeInterval.NORMAL;
                break;
            case "MAX":
                interval = AdBrixRm.AdBrixEventUploadTimeInterval.MAX;
                break;
            default:
                interval = AdBrixRm.AdBrixEventUploadTimeInterval.NORMAL;
                break;
        }
        AdBrixRm.setEventUploadTimeInterval(interval);
    }

    private void setPushEnable(MethodCall call) {
        Boolean enable = call.arguments();
        AdBrixRm.setPushEnable(enable);
    }

    private void setRegistrationId(MethodCall call) {
        String value = call.arguments();
        AdBrixRm.setRegistrationId(value);
    }

    private void setNotificationChannel(MethodCall call) {
        String channelName = call.argument("channelName");
        String channelDescription = call.argument("channelDescription");
        AdBrixRm.createDefaultNotificationChannel(channelName, channelDescription);
    }

    private void fetchInAppMessage(final Result result) {
        AdBrixRm.fetchInAppMessage(new Completion<io.adbrix.sdk.domain.model.Result<Empty>>() {
            @Override
            public void handle(io.adbrix.sdk.domain.model.Result<Empty> emptyResult) {
                result.success(emptyResult.isSucceeded());
            }
        });
    }

    private void getAllInAppMessage(final Result result) {
        AdBrixRm.getAllInAppMessage(new Completion<io.adbrix.sdk.domain.model.Result<List<DfnInAppMessage>>>() {
            @Override
            public void handle(io.adbrix.sdk.domain.model.Result<List<DfnInAppMessage>> listResult) {
                List<DfnInAppMessage> list = listResult.getOrDefault(new ArrayList<DfnInAppMessage>());
                JSONArray array = new JSONArray();
                for (DfnInAppMessage index : list) {
                    if (index == null) {
                        continue;
                    }
                    try {
                        array.put(index);
                    } catch (Exception e) {
                        AbxLog.e("getAllInAppMessage() error", e, false);
                    }
                }
                if (listResult.isSucceeded()) {
                    result.success(array.toString());
                } else {
                    result.error(ERROR_CODE_SDK_ERROR, listResult.toString(), null);
                }
            }
        });
    }

    private void openInAppMessage(MethodCall call, final Result result) {
        String campaignId = call.argument("campaignId");
        AdBrixRm.openInAppMessage(campaignId, new Completion<io.adbrix.sdk.domain.model.Result<Empty>>() {
            @Override
            public void handle(io.adbrix.sdk.domain.model.Result<Empty> emptyResult) {
                result.success(emptyResult.isSucceeded());
            }
        });
    }
    // endregion

    // region AbxCommon

    private void commonPurchase(MethodCall call) {

        String orderId = call.argument("orderId");
        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        Double discount = call.argument("discount");
        Double orderSale = call.argument("orderSale");
        Double deliveryCharge = call.argument("deliveryCharge");
        String paymentMethod = call.argument("paymentMethod");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null && orderId != null && orderSale != null && discount != null && deliveryCharge != null
                && paymentMethod != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AdBrixRm.CommonProperties.Purchase purchaseAttr = new AdBrixRm.CommonProperties.Purchase()
                        .setAttrModel(evetAttr);
                AbxCommon.purchase(orderId, productModelList, orderSale, discount, deliveryCharge,
                        AdBrixRm.CommercePaymentMethod.getMethodByMethodCode(paymentMethod), purchaseAttr);
            } else {
                AbxCommon.purchase(orderId, productModelList, orderSale, discount, deliveryCharge,
                        AdBrixRm.CommercePaymentMethod.getMethodByMethodCode(paymentMethod));
            }
        }
    }

    private void commonSignUp(MethodCall call) {

        Integer channel = call.argument("channel");
        HashMap<String, Object> attr = call.argument("attr");

        if (channel != null) {

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.CommonProperties.SignUp signUpUserInfo = new AdBrixRm.CommonProperties.SignUp()
                        .setAttrModel(eventAttr);
                AbxCommon.signUp(AdBrixRm.CommonSignUpChannel.getChannelByChannelCode(channel), signUpUserInfo);

            } else {
                AbxCommon.signUp(AdBrixRm.CommonSignUpChannel.getChannelByChannelCode(channel));

            }
        }
    }

    private void commonUseCredit(MethodCall call) {

        HashMap<String, Object> attr = call.argument("attr");

        if (attr != null) {
            AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
            AdBrixRm.CommonProperties.UseCredit useCreditInfo = new AdBrixRm.CommonProperties.UseCredit()
                    .setAttrModel(eventAttr);
            AbxCommon.useCredit(useCreditInfo);

        } else {
            AbxCommon.useCredit();

        }
    }

    private void commonAppUpdate(MethodCall call) {

        String preVersion = call.argument("preVersion");
        String currVersion = call.argument("currVersion");
        HashMap<String, Object> attr = call.argument("attr");

        if (attr != null) {
            AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
            AdBrixRm.CommonProperties.AppUpdate updateInfo = new AdBrixRm.CommonProperties.AppUpdate()
                    .setPrevVersion(preVersion)
                    .setCurrVersion(currVersion)
                    .setAttrModel(eventAttr);
            AbxCommon.appUpdate(updateInfo);

        } else {
            AdBrixRm.CommonProperties.AppUpdate updateInfo = new AdBrixRm.CommonProperties.AppUpdate()
                    .setPrevVersion(preVersion)
                    .setCurrVersion(currVersion);
            AbxCommon.appUpdate(updateInfo);

        }
    }

    private void commonInvite(MethodCall call) {

        Integer inviteChannel = call.argument("inviteChannel");
        HashMap<String, Object> attr = call.argument("attr");

        if (inviteChannel != null) {

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.CommonProperties.Invite inviteInfo = new AdBrixRm.CommonProperties.Invite()
                        .setAttrModel(eventAttr);
                AbxCommon.invite(AdBrixRm.CommonInviteChannel.getChannelByChannelCode(inviteChannel), inviteInfo);

            } else {
                AbxCommon.invite(AdBrixRm.CommonInviteChannel.getChannelByChannelCode(inviteChannel));

            }
        }
    }
    // endregion

    // region AbxCommerce
    private void commerceViewHome(MethodCall call) {
        HashMap<String, Object> attr = call.argument("attr");
        if (attr != null) {
            AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
            AbxCommerce.viewHome(eventAttr);
        } else {
            AbxCommerce.viewHome();
        }
    }

    private void commerceCategoryView(MethodCall call) {
        ArrayList<String> categoryModel = call.argument("categoryModel");
        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        HashMap<String, Object> attr = call.argument("attr");
        if (productList != null && categoryModel != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);
            AdBrixRm.CommerceCategoriesModel categoriesModel = new AdBrixRm.CommerceCategoriesModel();
            for (int i = 0, n = categoryModel.size(); i < n; i++) {
                String categoryListName = categoryModel.get(i);
                categoriesModel.setCategory(categoryListName);
            }
            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.categoryView(categoriesModel, productModelList, evetAttr);
            } else {
                AbxCommerce.categoryView(categoriesModel, productModelList);
            }
        }
    }

    private void commerceProductView(MethodCall call) {

        HashMap<String, Object> productModel = call.argument("productModel");
        HashMap<String, Object> attr = call.argument("attr");

        if (productModel != null) {
            AdBrixRm.CommerceProductModel userProduct = Utils.makeProductModel(productModel);

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AbxCommerce.productView(userProduct, eventAttr);

            } else {
                AbxCommerce.productView(userProduct);
            }

        }

    }

    private void commerceAddToCart(MethodCall call) {

        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.addToCart(productModelList, evetAttr);

            } else {
                AbxCommerce.addToCart(productModelList);

            }
        }
    }

    private void commerceAddToWishList(MethodCall call) {

        HashMap<String, Object> productModel = call.argument("productModel");
        HashMap<String, Object> attr = call.argument("attr");

        if (productModel != null) {
            AdBrixRm.CommerceProductModel userProduct = Utils.makeProductModel(productModel);

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AbxCommerce.addToWishList(userProduct, eventAttr);

            } else {
                AbxCommerce.addToWishList(userProduct);
            }
        }
    }

    private void commerceReviewOrder(MethodCall call) {

        String orderId = call.argument("orderId");
        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        Double discount = call.argument("discount");
        Double deliveryCharge = call.argument("deliveryCharge");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null && orderId != null && discount != null && deliveryCharge != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.reviewOrder(orderId, productModelList, discount, deliveryCharge, evetAttr);

            } else {
                AbxCommerce.reviewOrder(orderId, productModelList, discount, deliveryCharge);

            }
        }
    }

    private void commerceRefund(MethodCall call) {

        String orderId = call.argument("orderId");
        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        Double penaltyCharge = call.argument("penaltyCharge");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null && orderId != null && penaltyCharge != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.refund(orderId, productModelList, penaltyCharge, evetAttr);

            } else {
                AbxCommerce.refund(orderId, productModelList, penaltyCharge);

            }
        }
    }

    private void commerceSearch(MethodCall call) {

        String keyword = call.argument("keyword");
        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null && keyword != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.search(keyword, productModelList, evetAttr);

            } else {
                AbxCommerce.search(keyword, productModelList);

            }
        }
    }

    private void commerceShare(MethodCall call) {

        HashMap<String, Object> productModel = call.argument("productModel");
        HashMap<String, Object> attr = call.argument("attr");
        String sharingChannel = call.argument("sharingChannel");

        if (productModel != null && sharingChannel != null) {
            AdBrixRm.CommerceProductModel userProduct = Utils.makeProductModel(productModel);
            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AbxCommerce.share(AdBrixRm.CommerceSharingChannel.getChannelByChannelCode(sharingChannel), userProduct,
                        eventAttr);
            } else {
                AbxCommerce.share(AdBrixRm.CommerceSharingChannel.getChannelByChannelCode(sharingChannel), userProduct);
            }
        }
    }

    private void commerceListView(MethodCall call) {

        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.listView(productModelList, evetAttr);

            } else {
                AbxCommerce.listView(productModelList);

            }
        }
    }

    private void commerceCartView(MethodCall call) {

        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.cartView(productModelList, evetAttr);

            } else {
                AbxCommerce.cartView(productModelList);

            }
        }
    }

    private void commercePaymentInfoAdd(MethodCall call) {
        HashMap<String, Object> attr = call.argument("attr");
        if (attr != null) {
            AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
            AbxCommerce.paymentInfoAdded(eventAttr);
        } else {
            AbxCommerce.paymentInfoAdded();
        }
    }
    // endregion

    // region AbxGame
    private void gameTutorialComplete(MethodCall call) {

        Boolean isSkip = call.argument("isSkip");
        HashMap<String, Object> attr = call.argument("attr");

        if (isSkip != null) {

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.GameProperties.TutorialComplete gameProperties = new AdBrixRm.GameProperties.TutorialComplete()
                        .setIsSkip(isSkip)
                        .setAttrModel(eventAttr);
                AbxGame.tutorialComplete(gameProperties);

            } else {
                AdBrixRm.GameProperties.TutorialComplete gameProperties = new AdBrixRm.GameProperties.TutorialComplete()
                        .setIsSkip(isSkip);
                AbxGame.tutorialComplete(gameProperties);

            }

        }

    }

    private void gameLevelAchieved(MethodCall call) {
        Integer levelAchieved = call.argument("levelAchieved");
        HashMap<String, Object> attr = call.argument("attr");
        if (levelAchieved != null) {
            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.GameProperties.LevelAchieved gameProperties = new AdBrixRm.GameProperties.LevelAchieved()
                        .setLevel(levelAchieved)
                        .setAttrModel(eventAttr);
                AbxGame.levelAchieved(gameProperties);
            } else {
                AdBrixRm.GameProperties.LevelAchieved gameProperties = new AdBrixRm.GameProperties.LevelAchieved()
                        .setLevel(levelAchieved);
                AbxGame.levelAchieved(gameProperties);
            }
        }
    }

    private void gameCharacterCreated(MethodCall call) {

        HashMap<String, Object> attr = call.argument("attr");

        if (attr != null) {
            AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
            AdBrixRm.GameProperties.CharacterCreated gameProperties = new AdBrixRm.GameProperties.CharacterCreated()
                    .setAttrModel(eventAttr);
            AbxGame.characterCreated(gameProperties);

        } else {
            AbxGame.characterCreated();

        }
    }

    private void gameStageCleared(MethodCall call) {

        String stageName = call.argument("stageName");
        HashMap<String, Object> attr = call.argument("attr");

        if (stageName != null) {

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.GameProperties.StageCleared gameProperties = new AdBrixRm.GameProperties.StageCleared()
                        .setStageName(stageName)
                        .setAttrModel(eventAttr);
                AbxGame.stageCleared(gameProperties);

            } else {
                AdBrixRm.GameProperties.StageCleared gameProperties = new AdBrixRm.GameProperties.StageCleared()
                        .setStageName(stageName);
                AbxGame.stageCleared(gameProperties);

            }

        }

    }

    private void setSubscriptionStatus(MethodCall call, final Result result) {
        String info = call.argument("informative");
        String marketing = call.argument("marketing");
        String marketing_push = call.argument("marketing_push");
        String marketing_sms = call.argument("marketing_sms");
        String marketing_kakao = call.argument("marketing_kakao");
        String marketing_night = call.argument("marketing_night");
        String marketing_night_push = call.argument("marketing_night_push");
        String marketing_night_sms = call.argument("marketing_night_sms");
        String marketing_night_kakao = call.argument("marketing_night_kakao");

        SubscriptionStatus status = new SubscriptionStatus.Builder()
                .setInformativeNotificationFlag(SubscriptionStatus.Type.valueOf(info))
                .setMarketingNotificationFlag(SubscriptionStatus.Type.valueOf(marketing))
                .setMarketingNotificationFlagForPushChannel(SubscriptionStatus.Type.valueOf(marketing_push))
                .setMarketingNotificationFlagForSmsChannel(SubscriptionStatus.Type.valueOf(marketing_sms))
                .setMarketingNotificationFlagForKakaoChannel(SubscriptionStatus.Type.valueOf(marketing_kakao))
                .setMarketingNotificationAtNightFlag(SubscriptionStatus.Type.valueOf(marketing_night))
                .setMarketingNotificationAtNightFlagForPushChannel(SubscriptionStatus.Type.valueOf(marketing_night_push))
                .setMarketingNotificationAtNightFlagForSmsChannel(SubscriptionStatus.Type.valueOf(marketing_night_sms))
                .setMarketingNotificationAtNightFlagForKakaoChannel(SubscriptionStatus.Type.valueOf(marketing_night_kakao))
                .build();

        AdBrixRm.setSubscriptionStatus(status, new AdBrixRm.SetSubscriptionStatusCallback() {
            @Override
            public void onCallback(SetSubscriptionStatusResult setSubscriptionStatusResult) {
                if (setSubscriptionStatusResult.isSuccess()) {
                    result.success(null);
                } else {
                    result.error(Integer.toString(setSubscriptionStatusResult.getResultCode()) , setSubscriptionStatusResult.getResultMessage(), null);
                }
            }
        });

    }

    private void getSubscriptionStatus(final Result result) {
        AdBrixRm.getSubscriptionStatus(new AdBrixRm.GetSubscriptionStatusCallback() {
            @Override
            public void onCallback(GetSubscriptionStatusResult getSubscriptionStatusResult) {
                if (getSubscriptionStatusResult.isSuccess()) {
                    HashMap<String, String> resultMap = new HashMap<>();
                    resultMap.put("informative", getSubscriptionStatusResult.getInformativeNotificationFlag().toString());
                    resultMap.put("marketing", getSubscriptionStatusResult.getMarketingNotificationFlag().toString());
                    resultMap.put("marketing_push", getSubscriptionStatusResult.getMarketingNotificationFlagForPushChannel().toString());
                    resultMap.put("marketing_sms", getSubscriptionStatusResult.getMarketingNotificationFlagForSmsChannel().toString());
                    resultMap.put("marketing_kakao", getSubscriptionStatusResult.getMarketingNotificationFlagForKakaoChannel().toString());
                    resultMap.put("marketing_night", getSubscriptionStatusResult.getMarketingNotificationAtNightFlag().toString());
                    resultMap.put("marketing_night_push", getSubscriptionStatusResult.getMarketingNotificationAtNightFlagForPushChannel().toString());
                    resultMap.put("marketing_night_sms", getSubscriptionStatusResult.getMarketingNotificationAtNightFlagForSmsChannel().toString());
                    resultMap.put("marketing_night_kakao", getSubscriptionStatusResult.getMarketingNotificationAtNightFlagForKakaoChannel().toString());
                    result.success(resultMap);
                } else {
                    result.error(Integer.toString(getSubscriptionStatusResult.getResultCode()) , getSubscriptionStatusResult.getResultMessage(), null);
                }
            }
        });
    }
    private void setKakoId(MethodCall call, final Result result){
        String kakaoId = call.arguments();

        AdBrixRm.setKakaoId(kakaoId, new AdBrixRm.SetCiProfileCallback() {
            @Override
            public void onCallback(SetCiProfileResult setCiProfileResult) {
                if (setCiProfileResult.isSuccess()) {
                    result.success(null);
                } else {
                    result.error(Integer.toString(setCiProfileResult.getResultCode()), setCiProfileResult.getResultMessage(), null);
                }
            }
        });
    }
    private void setPhoneNumber(MethodCall call, final Result result){
        String number = call.arguments();

        AdBrixRm.setPhoneNumber(number, new AdBrixRm.SetCiProfileCallback() {
            @Override
            public void onCallback(SetCiProfileResult setCiProfileResult) {
                if (setCiProfileResult.isSuccess()) {
                    result.success(null);
                } else {
                    result.error(Integer.toString(setCiProfileResult.getResultCode()), setCiProfileResult.getResultMessage(), null);
                }
            }
        });
    }

    // endregion
}
