import Flutter
import UIKit
import AdBrixRmKit

@objc
public class AdbrixrmFlutterPlugin: NSObject, FlutterPlugin {
    
    static let METHOD_CHANNEL = "com.adbrix.methodChannel"
    
    static let EVENT_CHANNEL_Deeplink = "com.adbrix.eventChannel.deeplink"
    static let EVENT_CHANNEL_DeferredDeeplink = "com.adbrix.eventChannel.deferredDeeplink"
    static let EVENT_CHANNEL_PushRemoteCallback = "com.adbrix.eventChannel.pushRemoteCallback"
    static let EVENT_CHANNEL_OnReceiveInAppMessageClick = "com.adbrix.eventChannel.onReceiveInAppMessageClick"
    
    let ERROR_CODE_SDK_ERROR = "-1";
    
    let adBrix = AdBrixRM.getInstance
    
    @objc
    public static func sdkInit(appKey: String, secretKey: String) {
        AdBrixRM.getInstance.initAdBrix(appKey: appKey, secretKey: secretKey)
    }
    
    ///
    /// You should call setDeeplinkListener() first if you want to set deeplink Listener for flutter
    @objc
    public static func deeplinkOpen(with url: URL) {
        AdBrixRM.getInstance.deepLinkOpen(url: url)
    }

    @objc
    public static func setDeeplinkListener() {
        AdBrixRM.getInstance.setDeeplinkDelegate(delegate: AdBrixEventChannel.shared)
    }

    
    @objc
    public static func setDeferredDeeplinkListener() {
        AdBrixRM.getInstance.setDeferredDeeplinkDelegate(delegate: AdBrixEventChannel.shared)
    }
    
    @objc
    public static func setRemotePushMessageListener() {
        AdBrixRM.getInstance.setAdBrixRmPushRemoteDelegate(delegate: AdBrixEventChannel.shared)
    }
    
    @objc
    public static func remotePushOpen(center: UNUserNotificationCenter, response: UNNotificationResponse) {
        AdBrixRM.getInstance.userNotificationCenter(center: center, response: response)
    }

    @objc
    public static func willPresentRemotePush(notification: UNNotification, completionHandler: @escaping (UNNotificationPresentationOptions) -> ()) {
        if let apsInfo = notification.request.content.userInfo["aps"] as? [String : Any] {
            if let genWhileRun = apsInfo["gen_while_run"] as? String,
                genWhileRun == "1" {
                completionHandler([.alert, .sound])
            }
        }
        completionHandler([])

    }
    
    @objc
    public static func setInAppMessageClickListener() {
        AdBrixRM.getInstance.setInAppMessageClickDelegate(delegate: AdBrixEventChannel.shared)
    }
    
    @objc
    public static func startGettingIDFA () {
        AdBrixRM.getInstance.startGettingIDFA();
    }
    
    @objc
    public static func stopGettingIDFA () {
        AdBrixRM.getInstance.stopGettingIDFA();
    }
    
    @objc
    public static func setPushEnable(toPushEnable: Bool) {
        AdBrixRM.getInstance.setPushEnable(toPushEnable: toPushEnable)
    }
    
    @objc
    public static func setRegistrationId(deviceToken: Data) {
        AdBrixRM.getInstance.setRegistrationId(deviceToken: deviceToken)
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = AdbrixrmFlutterPlugin()
        
        let methodChannel = FlutterMethodChannel(name: METHOD_CHANNEL, binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(instance, channel: methodChannel)
        
        [EVENT_CHANNEL_Deeplink,
         EVENT_CHANNEL_DeferredDeeplink,
         EVENT_CHANNEL_PushRemoteCallback,
         EVENT_CHANNEL_OnReceiveInAppMessageClick]
            .forEach {
                FlutterEventChannel(name: $0, binaryMessenger: registrar.messenger())
                    .setStreamHandler(AdBrixEventChannel.shared)
            }
    }
    
    //MethodChannel은 FlutterPlugin에 종속적이여서 이벤트채널처럼 다른 클래스로 분리하지 않았음
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        let methodName : String = call.method
        // event Controller
        
        switch methodName {
        case "setEventUploadCountInterval" :
            setEventUploadCountInterval (call)
        case"setEventUploadTimeInterval" :
            setEventUploadTimeInterval (call)
        case "login" :
            login (call)
        case "logout" :
            logout()
        case "setAge":
            setAge (call)
        case "setGender":
            setGender (call)
        case "setUserProperties" :
            setUserProperties (call)
        case "commonSignUp":
            commonSignUp (call)
        case "appUpdate" :
            commonAppUpdate (call)
        case "userInvite" :
            commonUserInvite (call)
        case "useCredit" :
            commonUseCredit (call)
        case "commonPurchase" :
            commonPurchase (call)
        case "event" :
            events (call)
        case "commerceViewHome" :
            commerceViewHome (call)
        case "commerceCategoryView" :
            commerceCategoryView(call)
        case "commerceProductView" :
            commerceProductView (call)
        case "commerceAddToCart" :
            commerceAddToCart (call)
        case "commerceAddToWishList" :
            commerceAddToWishList (call)
        case "commerceReviewOrder" :
            commerceReviewOrder (call)
        case "commerceRefund" :
            commerceRefund (call)
        case "commerceSearch" :
            commerceSearch (call)
        case "commerceShare" :
            commerceShare (call)
        case "commerceListView" :
            commerceListView (call)
        case "commerceCartView" :
            commerceCartView (call)
        case "commercePaymentInfoAdd" :
            commercePaymentInfoAdd (call)
        case "gameTutorialComplete" :
            gameTutorialComplete (call)
        case "gameCharacterCreated" :
            gameCharacterCreated (call)
        case "gameStageCleared" :
            gameStageCleared (call)
        case "gameLevelAchieved":
            gameLevelAchieved (call)
        case "gdprForgetMe" :
            gdprForgetMe ()
        case "setRegistrationId":
            setRegistrationId(call)
        case "startGettingIDFA" :
            AdbrixrmFlutterPlugin.startGettingIDFA()
        case "stopGettingIDFA" :
            AdbrixrmFlutterPlugin.stopGettingIDFA()
        case "setPushEnable":
            setPushEnable(call)
        case "fetchInAppMessage":
            fetchInAppMessage(result)
        case "getAllInAppMessage":
            getAllInAppMessage(result)
        case "openInAppMessage":
            openInAppMessage(call, result)
        case "setSubscriptionStatus":
            setSubscriptionStatus(call, result)
        case "getSubscriptionStatus":
            getSubscriptionStatus(result)
        case "setKakaoId":
            setKakoId(call, result)
        case "setPhoneNumber":
            setPhoneNumber(call, result)
        default:
            print("AdBrixRm Flutter plugin")
        }
    }
}


//MARK: - MethodHandler
extension AdbrixrmFlutterPlugin {
    
    func setRegistrationId(_ call: FlutterMethodCall) {
        guard let token = call.arguments as? String else {
            return
        }
        let data = Data(token.utf8)
        adBrix.setRegistrationId(deviceToken: data)
    }
    func setEventUploadCountInterval (_ call: FlutterMethodCall) {
        
        guard let arg = call.arguments as? String else {
            return
        }
        
        switch arg {
        case "MIN":
            adBrix.setEventUploadCountInterval(AdBrixRM.AdBrixEventUploadCountInterval.MIN)
        case "NORMAL" :
            adBrix.setEventUploadCountInterval(AdBrixRM.AdBrixEventUploadCountInterval.NORMAL)
        case "MAX" :
            adBrix.setEventUploadCountInterval(AdBrixRM.AdBrixEventUploadCountInterval.MAX)
        default:
            print("AdBrixRm Flutter plugin")
        }
    }

    func setEventUploadTimeInterval (_ call: FlutterMethodCall) {
        
        guard let arg = call.arguments as? String else {
            return
        }
        print(arg)
        switch arg {
        case "MIN":
            adBrix.setEventUploadTimeInterval(AdBrixRM.AdBrixEventUploadTimeInterval.MIN)
        case "NORMAL" :
            adBrix.setEventUploadTimeInterval(AdBrixRM.AdBrixEventUploadTimeInterval.NORMAL)
        case "MAX" :
            adBrix.setEventUploadTimeInterval(AdBrixRM.AdBrixEventUploadTimeInterval.MAX)
        default:
            print("AdBrixRm Flutter plugin")
        }
    }
    
    func gdprForgetMe () {
        
        adBrix.gdprForgetMe()
    }
    
    func setAge (_ call : FlutterMethodCall) {
        
        guard let arg = call.arguments as? Int else {
            return
        }
        adBrix.setAge(int: arg)
    }
    
    func setGender (_ call : FlutterMethodCall) {
        
        guard let arg = call.arguments as? String else {
            return
        }
        switch arg {
        case "FEMALE":
            adBrix.setGender(adBrixGenderType: AdBrixRM.AdBrixGenderType.Female)
        case "MALE" :
            adBrix.setGender(adBrixGenderType: AdBrixRM.AdBrixGenderType.Male)
        case "UNKNOWN" :
            adBrix.setGender(adBrixGenderType: AdBrixRM.AdBrixGenderType.Unknown)
        default:
            print("AdBrixRm Flutter plugin")
        }
    }
    
    func setUserProperties (_ call : FlutterMethodCall) {
        
        guard let arg = (call.arguments as? [String : Any])?.convertBoolType() else {
            return
        }
        let userProperties = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: arg)
        adBrix.setUserPropertiesWithAttr(attrModel: userProperties)
        
    }
    
    func events (_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
        let eventName = args["eventName"] as? String else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.eventWithAttr(eventName: eventName, value: eventAttr)
        } else {
            adBrix.event(eventName: eventName)
        }
    }
    
    func login (_ call : FlutterMethodCall) {
        
        guard let userId = call.arguments as? String else {
            return
        }
        
        adBrix.login(userId: userId)
        
    }
    
    func logout (){
        adBrix.logout();
    }
    
    func commonSignUp(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let channel = args["channel"] as? Int,
              let adbrixChannel = AdBrixRM.AdBrixRmSignUpChannel(rawValue: channel) else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commonSignUpWithAttr(channel: adbrixChannel, commonAttr: eventAttr)
        } else {
            adBrix.commonSignUp(channel: adbrixChannel)
            
        }
    }
    
    func commonAppUpdate(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let preVersion = args["preVersion"] as? String,
              let currVersion = args["currVersion"] as? String else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commonAppUpdateWithAttr(prev_ver: preVersion, curr_ver: currVersion, commonAttr: eventAttr)
        }else {
            adBrix.commonAppUpdate(prev_ver: preVersion, curr_ver: currVersion)
            
        }
    }
    
    func commonUserInvite(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let channel = args["inviteChannel"] as? Int,
              let adbrixChannel = AdBrixRM.AdBrixRmInviteChannel(rawValue: channel)
        else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commonInviteWithAttr(channel: adbrixChannel, commonAttr: eventAttr)
        } else {
            adBrix.commonInvite(channel: adbrixChannel)
            
        }
    }
    
    func commonUseCredit(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType() else {
            adBrix.commonUseCredit()
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commonUseCreditWithAttr(commonAttr: eventAttr)
        }
    }
    
    func gameTutorialComplete(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let isSkip = args["isSkip"] as? Bool else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.gameTutorialCompletedWithAttr(isSkip: isSkip, gameInfoAttr: eventAttr)
        } else {
            adBrix.gameTutorialCompleted(isSkip: isSkip)
        }
    }
    
    func gameCharacterCreated(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType() else {
            adBrix.gameCharacterCreated()
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.gameCharacterCreatedWithAttr(gameInfoAttr: eventAttr)
        }
    }
    
    func gameStageCleared(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let stageName = args["stageName"] as? String else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.gameStageClearedWithAttr(stageName: stageName, gameInfoAttr: eventAttr)
        } else {
            adBrix.gameStageCleared(stageName: stageName)
        }
    }
    
    func gameLevelAchieved(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let levelAchieved = args["levelAchieved"] as? Int else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.gameLevelAchievedWithAttr(level: levelAchieved, gameInfoAttr: eventAttr)
        } else {
            adBrix.gameLevelAchieved(level: levelAchieved)
        }
    }
    
    func commerceViewHome(_ call: FlutterMethodCall) {
        guard let args = (call.arguments as? [String : Any])?.convertBoolType() else {
            adBrix.commerceViewHome()
            return
        }

        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commerceViewHomeWithAttr(orderAttr: eventAttr)
        }
    }
    
    func commerceProductView(_ call : FlutterMethodCall) {
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let productModel = args["productModel"] as? [String: Any] else {
            return
        }
        
        let productResult = AdBrixUtility.AdBrixRmMakeProductModel(productModel: productModel)
        
        if let attr = args["attr"] as? [String: Any]{
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commerceProductViewWithAttr(productInfo: productResult, orderAttr: eventAttr)
            
        } else {
            adBrix.commerceProductView(productInfo:productResult)
            
        }
    }
    
    func commerceAddToWishList(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let productModel = args["productModel"] as? [String: Any] else {
            return
        }
        
        let productResult = AdBrixUtility.AdBrixRmMakeProductModel(productModel: productModel)
        
        if let attr = args["attr"] as? [String: Any]{
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commerceAddToWishListWithAttr(productInfo: productResult, orderAttr: eventAttr)
        } else {
            adBrix.commerceAddToWishList(productInfo: productResult)
        }
    }
    
    func commerceShare (_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let productModel = args["productModel"] as? [String: Any],
              let shareChannel = args["sharingChannel"] as? String else {
            return
        }
        
        
        let productResult = AdBrixUtility.AdBrixRmMakeProductModel(productModel: productModel)
        
        var sharingChannel: AdBrixRM.AdBrixRmSharingChannel?
        switch shareChannel {
        case "FACEBOOK":
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingFacebook
        case "KAKAOTALK" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingKakaoTalk
        case "KAKAOSTORY" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingKakaoStory
        case "LINE" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingLine
        case "WHATSAPP" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingWhatsApp
        case "QQ" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingQQ
        case "WECHAT" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingWeChat
        case "SMS" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingSMS
        case "EMAIL" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingEmail
        case "COPYURL" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingCopyUrl
        case "ETC" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingETC
        default:
            sharingChannel = nil
            debugPrint("peterworks :::::: there is something wrong with shareChannel")
        }
        
        guard let _sharingChannel = sharingChannel else {return}
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceShareWithAttr(channel: _sharingChannel, productInfo: productResult, orderAttr: eventAttr)
            
        }  else {
            adBrix.commerceShare(channel: _sharingChannel, productInfo: productResult)
            
        }
        
    }
    
    func commercePaymentInfoAdd (_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType() else {
            adBrix.commercePaymentInfoAdded()
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commercePaymentInfoAddedWithAttr(paymentInfoAttr: eventAttr)
        }
    }
    
    func commonPurchase (_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let orderId = args["orderId"] as? String,
              let orderSale = args["orderSale"] as? Double,
              let productList = args["productList"] as? [[String: Any]],
              let discount = args["discount"] as? Double,
              let deliveryCharge = args["deliveryCharge"] as? Double,
              let paymentMethodString = args["paymentMethod"] as? String else {
            return
        }
        
        var paymentMethod: AdBrixRM.AdbrixRmPaymentMethod?
        
        switch paymentMethodString {
        case "CreditCard":
            paymentMethod = AdBrixRM.AdbrixRmPaymentMethod.CreditCard
        case "BankTransfer" :
            paymentMethod = AdBrixRM.AdbrixRmPaymentMethod.BankTransfer
        case "MobilePayment" :
            paymentMethod = AdBrixRM.AdbrixRmPaymentMethod.MobilePayment
        case "ETC" :
            paymentMethod = AdBrixRM.AdbrixRmPaymentMethod.ETC
        default:
            paymentMethod = nil
            debugPrint("peterworks :::::: there is something wrong with payment")
        }
        
        guard let paymentMethod = paymentMethod else {return}
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commonPurchaseWithAttr(orderId: orderId, productInfo: productListArr, orderSales: orderSale, discount: discount, deliveryCharge: deliveryCharge, paymentMethod: paymentMethod, orderAttr: eventAttr)
            
        } else {
            
            adBrix.commonPurchase(orderId: orderId, productInfo: productListArr, orderSales: orderSale, discount: discount, deliveryCharge: deliveryCharge, paymentMethod: paymentMethod)
            
        }
        
    }
    
    func commerceCategoryView(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let productList = args["productList"] as? [[String: Any]],
              let categoryList = args["categoryModel"] as? [String] else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        let myCategory = AdBrixUtility.AdBrixRmMakeProductCategory(categoryList: categoryList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceCategoryViewWithAttr(category: myCategory, productInfo: productListArr, orderAttr: eventAttr)
            
        } else {
            
            adBrix.commerceCategoryView(category: myCategory, productInfo: productListArr)
        }
        
    }
    
    func commerceAddToCart(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let productList = args["productList"] as? [[String: Any]] else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceAddToCartWithAttr(productInfo: productListArr, orderAttr: eventAttr)
            
        } else {
            
            adBrix.commerceAddToCart(productInfo: productListArr)
        }
    }
    
    func commerceReviewOrder (_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let orderId = args["orderId"] as? String,
              let productList = args["productList"] as? [[String: Any]],
              let discount = args["discount"] as? Double,
              let deliveryCharge = args["deliveryCharge"] as? Double else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any]{
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceReviewOrderWithAttr(orderId: orderId, productInfo: productListArr, discount: discount, deliveryCharge: deliveryCharge, orderAttr: eventAttr)
            
        } else {
            
            adBrix.commerceReviewOrder(orderId: orderId, productInfo: productListArr, discount: discount, deliveryCharge: deliveryCharge)
            
        }
        
    }
    
    func commerceRefund (_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let orderId = args["orderId"] as? String,
              let productList = args["productList"] as? [[String: Any]],
              let penaltyCharge = args["penaltyCharge"] as? Double else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceRefundWithAttr(orderId: orderId, productInfo: productListArr, penaltyCharge: penaltyCharge, orderAttr: eventAttr)
            
        } else {
            
            adBrix.commerceRefund(orderId: orderId, productInfo: productListArr, penaltyCharge: penaltyCharge)
            
        }
        
    }
    
    func commerceSearch(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let keyword = args["keyword"] as? String,
              let productList = args["productList"] as? [[String: Any]] else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceSearchWithAttr(productInfo: productListArr, keyword: keyword, orderAttr: eventAttr)
            
        } else {
            adBrix.commerceSearch(productInfo: productListArr, keyword: keyword)
        }
    }
    
    func commerceListView(_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let productList = args["productList"] as? [[String: Any]] else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceListViewWithAttr(productInfo: productListArr, orderAttr: eventAttr)
            
        } else {
            adBrix.commerceListView(productInfo: productListArr)
        }
    }
    
    func commerceCartView (_ call : FlutterMethodCall) {
        
        guard let args = (call.arguments as? [String : Any])?.convertBoolType(),
              let productList = args["productList"] as? [[String: Any]] else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceCartViewWithAttr(productInfo: productListArr, orderAttr: eventAttr)
        } else {
            adBrix.commerceCartView(productInfo: productListArr)
        }
    }
    
    func setPushEnable(_ call : FlutterMethodCall) {
        guard let isPushEnable = call.arguments as? Bool else {
            NSLog("abxrm \(#function) [error] :: cannot parse arguments!, argument = \(call.arguments.debugDescription)")
            return
        }
        
        adBrix.setPushEnable(toPushEnable: isPushEnable)
    }
    
    func fetchInAppMessage(_ result: @escaping FlutterResult) {
        adBrix.fetchInAppMessage() { fetchResult in
            if fetchResult.isSucceeded {
                result(true)
            } else {
                result(false)
            }
        }
    }
    
    func getAllInAppMessage(_ result: @escaping FlutterResult) {
        adBrix.getAllInAppMessage() { fetchResult in
            if fetchResult.isSucceeded {
                let dataArray = fetchResult.getData() ?? []
                result(dataArray.map{ $0.campaignId }.description)
            } else {
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: fetchResult.getError()?.localizedDescription, details: nil))
            }
        }
    }
    
    func openInAppMessage(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let arg = call.arguments as? [String: Any?],
              let campaignId = arg["campaignId"] as? String
        else {
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        adBrix.openInAppMessage(campaignId: campaignId) { completion in
            switch completion {
            case .success:
                result(true)
            case .fail:
                result(false)
            @unknown default:
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: "unkown Completion Type", details: nil))
            }
        }
    }
    
    func setSubscriptionStatus(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let dict = call.arguments as? [String: Any?] else {
            return
        }
        
        guard let info = dict["informative"] as? String,
              let marketing = dict["marketing"] as? String,
              let marketingPush = dict["marketing_push"] as? String,
              let marketingSms = dict["marketing_sms"] as? String,
              let marketingKakao = dict["marketing_kakao"] as? String,
              let marketingNight = dict["marketing_night"] as? String,
              let marketingNightPush = dict["marketing_night_push"] as? String,
              let marketingNightSms = dict["marketing_night_sms"] as? String,
              let marketingNightKakao = dict["marketing_night_kakao"] as? String else {
            result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: "Wrong SubscriptionStatus value", details: nil))
            return
        }
        
        SubscriptionStatus
            .Builder()
            .setInformativeNotificationFlag(to: convert(info))
            .setMarketingNotificationFlag(to: convert(marketing))
            .setMarketingNotificationFlagForPushChannel(to: convert(marketingPush))
            .setMarketingNotificationFlagForSmsChannel(to: convert(marketingSms))
            .setMarketingNotificationFlagForKakaoChannel(to: convert(marketingKakao))
            .setMarketingNotificationAtNightFlag(to: convert(marketingNight))
            .setMarketingNotificationAtNightFlagForPushChannel(to: convert(marketingNightPush))
            .setMarketingNotificationAtNightFlagForSmsChannel(to: convert(marketingNightSms))
            .setMarketingNotificationAtNightFlagForKakaoChannel(to: convert(marketingNightKakao))
            .build()
            .send { setResult in
                guard setResult.isSuccess else {
                    result(FlutterError(code: "\(setResult.resultCode ?? -1)" , message: setResult.resultMessage, details: nil))
                    return
                }
                result(nil)
                return
            }
        
        func convert(_ status: String) -> SubscriptionStatus._Type {
            switch status {
            case "SUBSCRIBED":
                return .SUBSCRIBED
            case "UNSUBSCRIBED":
                return .UNSUBSCRIBED
            default:
                return .UNDEFINED
            }
        }
    }
    
    func getSubscriptionStatus(_ result: @escaping FlutterResult) {
        AdBrixRM.getInstance.getSubscriptionStatus { getResult in
            guard getResult.isSuccess,
                  let status = getResult.value else {
                result(FlutterError(code: "\(getResult.resultCode ?? -1)", message: getResult.resultMessage, details: nil))
                return
            }
            
            result(
                [
                    "informative": status.informativeNotificationFlag.toString,
                    "marketing": status.marketingNotificationFlag.toString,
                    "marketing_push": status.marketingNotificationFlagForPushChannel.toString,
                    "marketing_sms": status.marketingNotificationFlagForSmsChannel.toString,
                    "marketing_kakao": status.marketingNotificationFlagForKakaoChannel.toString,
                    "marketing_night": status.marketingNotificationAtNightFlag.toString,
                    "marketing_night_push": status.marketingNotificationAtNightFlagForPushChannel.toString,
                    "marketing_night_sms": status.marketingNotificationAtNightFlagForSmsChannel.toString,
                    "marketing_night_kakao": status.marketingNotificationAtNightFlagForKakaoChannel.toString
                ]
            )
            
        }
    
    }

    func setKakoId(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let kakaoId = call.arguments as? String else {
            result(FlutterError(code: ERROR_CODE_SDK_ERROR, message: nil, details: nil))
            return
        }
        
        adBrix.setKakaoId(kakaoId: kakaoId) { setResult in
            guard setResult.isSuccess else {
                result(FlutterError(code: "\(setResult.resultCode ?? -1)", message: setResult.resultMessage, details: nil))
                return
            }
            result(nil)
        }
    }
    
    func setPhoneNumber(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let number = call.arguments as? String else {
            result(FlutterError(code: ERROR_CODE_SDK_ERROR, message: nil, details: nil))
            return
        }
        adBrix.setPhoneNumber(number: number) { setResult in
            guard setResult.isSuccess else {
                result(FlutterError(code: "\(setResult.resultCode ?? -1)", message: setResult.resultMessage, details: nil))
                return
            }
            
            result(nil)
        }
    }
    
}

extension SubscriptionStatus._Type {
    var toString: String {
        switch self {
        case .SUBSCRIBED:
            return "SUBSCRIBED"
        case .UNSUBSCRIBED:
            return "UNSUBSCRIBED"
        case .UNDEFINED:
            return "UNDEFINED"
        @unknown default:
            return "UNDEFINED"
        }
    }
}
