import 'dart:async';
import 'dart:io';

import 'package:flutter/services.dart';

class AdBrixRm {
  static const MethodChannel _methodChannel =
      const MethodChannel('com.adbrix.methodChannel');
  static const EventChannel _eventChannelDeeplink =
      const EventChannel('com.adbrix.eventChannel.deeplink');
  static const EventChannel _eventChannelDeferredDeeplink =
      const EventChannel('com.adbrix.eventChannel.deferredDeeplink');
  static const EventChannel _eventChannelPushRemoteCallback =
      const EventChannel('com.adbrix.eventChannel.pushRemoteCallback');
  static const EventChannel _eventChannelOnReceiveInAppMessageClick =
      const EventChannel('com.adbrix.eventChannel.onReceiveInAppMessageClick');

  static const String _ARGUMENT_Deeplink = 'com.adbrix.stream.deeplink';
  static const String _ARGUMENT_DeferredDeeplink =
      'com.adbrix.stream.deferredDeeplink';
  static const String _ARGUMENT_PushRemoteCallback =
      'com.adbrix.stream.pushRemoteCallback';
  static const String _ARGUMENT_OnReceiveInAppMessageClick =
      'com.adbrix.stream.onReceiveInAppMessageClick';

  static void setDeeplinkListener(Function(String deeplinkString) callback) {
    _eventChannelDeeplink
        .receiveBroadcastStream(_ARGUMENT_Deeplink)
        .listen((event) {
      callback(event.toString());
    });
  }

  static void setDeferredDeeplinkListener(
      Function(String deferredDeeplinkString) callback) {
    _eventChannelDeferredDeeplink
        .receiveBroadcastStream(_ARGUMENT_DeferredDeeplink)
        .listen((event) {
      callback(event.toString());
    });
  }

  static void setPushRemoteCallbackListener(
      Function(String jsonString) callback) {
    _eventChannelPushRemoteCallback
        .receiveBroadcastStream(_ARGUMENT_PushRemoteCallback)
        .listen((event) {
      callback(event.toString());
    });
  }

  static void setOnReceiveInAppMessageClickListener(
      Function(String jsonString) callback) {
    _eventChannelOnReceiveInAppMessageClick
        .receiveBroadcastStream(_ARGUMENT_OnReceiveInAppMessageClick)
        .listen((event) {
      callback(event.toString());
    });
  }

  static void setEventUploadCountInterval(
      {required AdBrixEventUploadCountInterval interval}) {
    _methodChannel.invokeMethod(
        _AbxAPI.setEventUploadCountInterval.toString().split('.').last,
        interval.toString().split('.').last);
  }

  static void setEventUploadTimeInterval(
      {required AdBrixEventUploadTimeInterval interval}) {
    _methodChannel.invokeMethod(
        _AbxAPI.setEventUploadTimeInterval.toString().split('.').last,
        interval.toString().split('.').last);
  }

  static void setRegistrationId({required String token}) {
    _methodChannel.invokeMethod(
        _AbxAPI.setRegistrationId.toString().split('.').last, token);
  }

  //Event
  static void login({required String userId}) {
    _methodChannel.invokeMethod(
        _AbxAPI.login.toString().split('.').last, userId);
  }

  static void logout() {
    _methodChannel.invokeMethod(_AbxAPI.logout.toString().split('.').last);
  }

  static void setAge({required int age}) {
    _methodChannel.invokeMethod(_AbxAPI.setAge.toString().split('.').last, age);
  }

  static void setGender({required AdBrixGender gender}) {
    _methodChannel.invokeMethod(_AbxAPI.setGender.toString().split('.').last,
        gender.toString().split('.').last);
  }

  static void setUserProperties({required Map<String, dynamic> properties}) {
    _attrBoolChanger(properties);
    _invokeWithAttr(
        _AbxAPI.setUserProperties.toString().split('.').last, properties, null);
  }

  static void commonSignUp(
      {required AdBrixSignUpChannel channel, Map<String, dynamic>? attr}) {
    int channelValue = channel.intVal;

    Map<String, dynamic> params = {'channel': channelValue};

    _invokeWithAttr(
        _AbxAPI.commonSignUp.toString().split('.').last, params, attr);
  }

  static void commonAppUpdate(
      {required String preVersion,
      required String currVersion,
      Map<String, dynamic>? attr}) {
    Map<String, dynamic> params = {
      'preVersion': preVersion,
      'currVersion': currVersion,
    };
    _invokeWithAttr(_AbxAPI.appUpdate.toString().split('.').last, params, attr);
  }

  static void commonUserInvite(
      {required AdBrixInviteChannel inviteChannel,
      Map<String, dynamic>? attr}) {
    Map<String, dynamic> params = {
      'inviteChannel': inviteChannel.intVal,
    };

    _invokeWithAttr(
        _AbxAPI.userInvite.toString().split('.').last, params, attr);
  }

  static void commonUseCredit({Map<String, dynamic>? attr}) {
    _invokeWithAttr(_AbxAPI.useCredit.toString().split('.').last, null, attr);
  }

  static void commonPurchase(
      {required String orderId,
      required List<AdBrixRmCommerceProductModel> productList,
      required double orderSale,
      required double discount,
      required double deliveryCharge,
      required AdBrixPaymentMethod paymentMethod,
      Map<String, dynamic>? attr}) {
    var getProductList = productList.map((e) => e.getProductModel()).toList();

    Map<String, dynamic> params = {
      'orderId': orderId,
      'productList': getProductList,
      'orderSale': orderSale,
      'discount': discount,
      'deliveryCharge': deliveryCharge,
      'paymentMethod': paymentMethod.toString().split('.').last
    };

    _invokeWithAttr(
        _AbxAPI.commonPurchase.toString().split('.').last, params, attr);
  }

  @Deprecated("Use 'event()'")
  static void events(
      {required String eventName, Map<String, dynamic>? attr}) async {
    Map<String, dynamic> params = {
      'eventName': eventName,
    };

    _invokeWithAttr(_AbxAPI.event.toString().split('.').last, params, attr);
  }

  static void event({required String eventName, Map<String, dynamic>? attr}) {
    Map<String, dynamic> params = {
      'eventName': eventName,
    };
    _invokeWithAttr(_AbxAPI.event.toString().split('.').last, params, attr);
  }

  static void commerceViewHome({Map<String, dynamic>? attr}) {
    _invokeWithAttr(
        _AbxAPI.commerceViewHome.toString().split('.').last, null, attr);
  }

  static void commerceCategoryView(
      {required AdBrixRmCommerceCategoryModel categoryModel,
      required List<AdBrixRmCommerceProductModel> productList,
      Map<String, dynamic>? attr}) {
    var getProductList = productList.map((e) => e.getProductModel()).toList();

    Map<String, dynamic> params = {
      'categoryModel': categoryModel.getCategoryList(),
      'productList': getProductList,
    };

    _invokeWithAttr(
        _AbxAPI.commerceCategoryView.toString().split('.').last, params, attr);
  }

  static void commerceProductView(
      {required AdBrixRmCommerceProductModel productModel,
      Map<String, dynamic>? attr}) {
    Map<String, dynamic> params = {
      'productModel': productModel.getProductModel(),
    };

    _invokeWithAttr(
        _AbxAPI.commerceProductView.toString().split('.').last, params, attr);
  }

  static void commerceAddToCart(
      {required List<AdBrixRmCommerceProductModel> productList,
      Map<String, dynamic>? attr}) {
    var getProductList = productList.map((e) => e.getProductModel()).toList();

    Map<String, dynamic> params = {
      'productList': getProductList,
    };

    _invokeWithAttr(
        _AbxAPI.commerceAddToCart.toString().split('.').last, params, attr);
  }

  static void commerceAddToWishList(
      {required AdBrixRmCommerceProductModel productModel,
      Map<String, dynamic>? attr}) {
    Map<String, dynamic> params = {
      'productModel': productModel.getProductModel(),
    };

    _invokeWithAttr(
        _AbxAPI.commerceAddToWishList.toString().split('.').last, params, attr);
  }

  static void commerceReviewOrder(
      {required String orderId,
      required List<AdBrixRmCommerceProductModel> productList,
      required double discount,
      required double deliveryCharge,
      Map<String, dynamic>? attr}) {
    var getProductList = productList.map((e) => e.getProductModel()).toList();

    Map<String, dynamic> params = {
      'orderId': orderId,
      'productList': getProductList,
      'discount': discount,
      'deliveryCharge': deliveryCharge,
    };

    _invokeWithAttr(
        _AbxAPI.commerceReviewOrder.toString().split('.').last, params, attr);
  }

  static void commerceRefund(
      {required String orderId,
      required List<AdBrixRmCommerceProductModel> productList,
      required double penaltyCharge,
      Map<String, dynamic>? attr}) {
    var getProductList = productList.map((e) => e.getProductModel()).toList();

    Map<String, dynamic> params = {
      'orderId': orderId,
      'productList': getProductList,
      'penaltyCharge': penaltyCharge,
    };

    _invokeWithAttr(
        _AbxAPI.commerceRefund.toString().split('.').last, params, attr);
  }

  static void commerceSearch(
      {required String keyword,
      required List<AdBrixRmCommerceProductModel> productList,
      Map<String, dynamic>? attr}) {
    var getProductList = productList.map((e) => e.getProductModel()).toList();

    Map<String, dynamic> params = {
      'keyword': keyword,
      'productList': getProductList,
    };

    _invokeWithAttr(
        _AbxAPI.commerceSearch.toString().split('.').last, params, attr);
  }

  static void commerceShare(
      {required AdBrixSharingChannel sharingChannel,
      required AdBrixRmCommerceProductModel productModel,
      Map<String, dynamic>? attr}) {
    Map<String, dynamic> params = {
      'sharingChannel': sharingChannel.toString().split('.').last,
      'productModel': productModel.getProductModel(),
    };

    _invokeWithAttr(
        _AbxAPI.commerceShare.toString().split('.').last, params, attr);
  }

  static void commerceListView(
      {required List<AdBrixRmCommerceProductModel> productList,
      Map<String, dynamic>? attr}) {
    var getProductList = productList.map((e) => e.getProductModel()).toList();

    Map<String, dynamic> params = {
      'productList': getProductList,
    };

    _invokeWithAttr(
        _AbxAPI.commerceListView.toString().split('.').last, params, attr);
  }

  static void commerceCartView(
      {required List<AdBrixRmCommerceProductModel> productList,
      Map<String, dynamic>? attr}) {
    var getProductList = productList.map((e) => e.getProductModel()).toList();

    Map<String, dynamic> params = {
      'productList': getProductList,
    };

    _invokeWithAttr(
        _AbxAPI.commerceCartView.toString().split('.').last, params, attr);
  }

  static void commercePaymentInfoAdd({Map<String, dynamic>? attr}) {
    _invokeWithAttr(
        _AbxAPI.commercePaymentInfoAdd.toString().split('.').last, null, attr);
  }

  static void gameTutorialComplete(
      {required bool isSkip, Map<String, dynamic>? attr}) {
    Map<String, dynamic> param = {
      'isSkip': isSkip,
    };

    _invokeWithAttr(
        _AbxAPI.gameTutorialComplete.toString().split('.').last, param, attr);
  }

  static void gameCharacterCreated({Map<String, dynamic>? attr}) {
    _invokeWithAttr(
        _AbxAPI.gameCharacterCreated.toString().split('.').last, null, attr);
  }

  static void gameStageCleared({
    required String stageName,
    Map<String, dynamic>? attr,
  }) {
    Map<String, dynamic> params = {
      'stageName': stageName,
    };

    _invokeWithAttr(
        _AbxAPI.gameStageCleared.toString().split('.').last, params, attr);
  }

  static void gameLevelAchieved(
      {required int levelAchieved, Map<String, dynamic>? attr}) {
    Map<String, dynamic> params = {
      'levelAchieved': levelAchieved,
      'attr': attr,
    };

    _invokeWithAttr(
        _AbxAPI.gameLevelAchieved.toString().split('.').last, params, attr);
  }

  static void gdprForgetMe() {
    _methodChannel
        .invokeMethod(_AbxAPI.gdprForgetMe.toString().split('.').last);
  }

  static void startGettingIDFA() {
    _methodChannel
        .invokeMethod(_AbxAPI.startGettingIDFA.toString().split('.').last);
  }

  static void stopGettingIDFA() {
    _methodChannel
        .invokeMethod(_AbxAPI.stopGettingIDFA.toString().split('.').last);
  }

  static void setPushEnable({required bool isPushEnable}) {
    _methodChannel.invokeMethod(
        _AbxAPI.setPushEnable.toString().split('.').last, isPushEnable);
  }

  static Future<bool?> fetchInAppMessage() {
    return _methodChannel.invokeMethod<bool>(
        _AbxAPI.fetchInAppMessage.toString().split('.').last);
  }

  static Future<String?> getAllInAppMessage() {
    return _methodChannel.invokeMethod<String>(
        _AbxAPI.getAllInAppMessage.toString().split('.').last);
  }

  static Future<bool?> openInAppMessage({required String campaignId}) {
    Map<String, dynamic> argument = {"campaignId": campaignId};
    return _methodChannel.invokeMethod<bool>(
        _AbxAPI.openInAppMessage.toString().split('.').last, argument);
  }

  static void setNotificationChannel(
      String channelName, String channelDescription) {
    if (!Platform.isAndroid) {
      return;
    }

    Map<String, dynamic> params = {
      "channelName": channelName,
      "channelDescription": channelDescription
    };
    _methodChannel.invokeMethod(
        _AbxAPI.setNotificationChannel.toString().split('.').last, params);
  }

  static Future<void> setSubscriptionStauts(
      {required bool info,
      required bool marketing,
      required bool marketingPush,
      required bool marketingSms,
      required bool marketingKakao,
      required bool marketingNight,
      required bool marketingNightPush,
      required bool marketingNightSms,
      required bool marketingNightKakao}) async {
    var status = {
      "informative": info ? "SUBSCRIBED" : "UNSUBSCRIBED",
      "marketing": marketing ? "SUBSCRIBED" : "UNSUBSCRIBED",
      "marketing_push": marketingPush ? "SUBSCRIBED" : "UNSUBSCRIBED",
      "marketing_sms": marketingSms ? "SUBSCRIBED" : "UNSUBSCRIBED",
      "marketing_kakao": marketingKakao ? "SUBSCRIBED" : "UNSUBSCRIBED",
      "marketing_night": marketingNight ? "SUBSCRIBED" : "UNSUBSCRIBED",
      "marketing_night_push":
          marketingNightPush ? "SUBSCRIBED" : "UNSUBSCRIBED",
      "marketing_night_sms": marketingNightSms ? "SUBSCRIBED" : "UNSUBSCRIBED",
      "marketing_night_kakao":
          marketingNightKakao ? "SUBSCRIBED" : "UNSUBSCRIBED"
    };
    return _methodChannel.invokeMethod(
        _AbxAPI.setSubscriptionStatus.toString().split('.').last, status);
  }

  static Future<Map?> getSubscriptionStatus() async {
    return _methodChannel.invokeMethod<Map>(
        _AbxAPI.getSubscriptionStatus.toString().split('.').last);
  }

  static Future<void> setKakaoId({required String kakaoId}) async {
    return _methodChannel.invokeMethod<void>(
        _AbxAPI.setKakaoId.toString().split('.').last, kakaoId);
  }

  static Future<void> setPhoneNumber({required String phoneNumber}) async {
    return _methodChannel.invokeMethod<void>(
        _AbxAPI.setPhoneNumber.toString().split('.').last, phoneNumber);
  }

  static void _invokeWithAttr(String methodName, Map<String, dynamic>? param,
      Map<String, dynamic>? attr) {
    if (attr != null) {
      _attrBoolChanger(attr);
    }

    Map<String, dynamic> combined = {...(param ?? {})};

    if (attr != null) {
      combined['attr'] = attr;
    }

    _methodChannel.invokeMethod(methodName, combined);
  }

  static void _attrBoolChanger(Map<String, dynamic> attr) {
    if (Platform.isIOS) {
      attr.entries.forEach((e) {
        if (e.value.runtimeType == bool) {
          String boolValueString = e.value.toString();
          attr.update(e.key, (value) => "abx_bool_$boolValueString");
        }
      });
    }
  }
}

class AdBrixRmCommerceCategoryModel {
  String category1 = "";
  String category2 = "";
  String category3 = "";
  String category4 = "";
  String category5 = "";
  List<String> _getList = [];

  AdBrixRmCommerceCategoryModel(String category1, String? category2,
      String? category3, String? category4, String? category5) {
    if ((category2 != null) &&
        (category3 != null) &&
        (category4 != null) &&
        (category5 != null)) {
      this.category1 = category1;
      this.category2 = category2;
      this.category3 = category3;
      this.category4 = category4;
      this.category5 = category5;

      this.setCategoryList(
          [category1, category2, category3, category4, category5]);
    } else if ((category2 != null) &&
        (category3 != null) &&
        (category4 != null)) {
      this.category1 = category1;
      this.category2 = category2;
      this.category3 = category3;
      this.category4 = category4;

      this.setCategoryList([category1, category2, category3, category4]);
    } else if ((category2 != null) && (category3 != null)) {
      this.category1 = category1;
      this.category2 = category2;
      this.category3 = category3;

      this.setCategoryList([category1, category2, category3]);
    } else if ((category2 != null)) {
      this.category1 = category1;
      this.category2 = category2;

      this.setCategoryList([category1, category2]);
    } else {
      this.category1 = category1;

      this.setCategoryList([category1]);
    }
  }

  List<String> getCategoryList() {
    return _getList;
  }

  void setCategoryList(List<String> getList) {
    this._getList = getList;
  }

  static AdBrixRmCommerceCategoryModel create(
      {required String category1,
      String? category2,
      String? category3,
      String? category4,
      String? category5}) {
    return new AdBrixRmCommerceCategoryModel(
        category1, category2, category3, category4, category5);
  }
}

class AdBrixRmCommerceProductModel {
  String productId = "";
  String productName = "";
  double price = 0.0;
  double discount = 0.0;
  int quantity = 0;
  String currency = "";
  List<String> category = [];
  Map<String, dynamic> productAttr = {};

  Map<String, dynamic> _productParameter = {};

  AdBrixRmCommerceProductModel(
      {required String productId,
      required String productName,
      required double price,
      required double discount,
      required int quantity,
      required AdBrixCurrency currency,
      required AdBrixRmCommerceCategoryModel category,
      Map<String, dynamic>? productAttr}) {
    if (productAttr != null) {
      this.productId = productId;
      this.productName = productName;
      this.price = price;
      this.discount = discount;
      this.quantity = quantity;
      this.currency = currency.toString().split('.').last;
      this.category = category.getCategoryList();
      this.productAttr = productAttr;

      AdBrixRm._attrBoolChanger(productAttr);

      Map<String, dynamic> setProduct = {
        'productId': productId,
        'productName': productName,
        'price': price,
        'discount': discount,
        'quantity': quantity,
        'currency': currency.toString().split('.').last,
        'category': category.getCategoryList(),
        'productAttr': productAttr
      };

      this.setProductModel(setProduct);
    } else {
      this.productId = productId;
      this.productName = productName;
      this.price = price;
      this.discount = discount;
      this.quantity = quantity;
      this.currency = currency.toString().split('.').last;
      this.category = category.getCategoryList();

      Map<String, dynamic> setProduct = {
        'productId': productId,
        'productName': productName,
        'price': price,
        'discount': discount,
        'quantity': quantity,
        'currency': currency.toString().split('.').last,
        'category': category.getCategoryList(),
      };

      this.setProductModel(setProduct);
    }
  }

  Map<String, dynamic> getProductModel() {
    return _productParameter;
  }

  void setProductModel(Map<String, dynamic> setupProduct) {
    this._productParameter = setupProduct;
  }

  static AdBrixRmCommerceProductModel create(
      {required String productId,
      required String productName,
      required double price,
      required double discount,
      required int quantity,
      required AdBrixCurrency currency,
      required AdBrixRmCommerceCategoryModel category,
      Map<String, dynamic>? productAttr}) {
    return new AdBrixRmCommerceProductModel(
        productId: productId,
        productName: productName,
        price: price,
        discount: discount,
        quantity: quantity,
        currency: currency,
        category: category,
        productAttr: productAttr);
  }
}

enum AdBrixSharingChannel {
  FACEBOOK,
  KAKAOTALK,
  KAKAOSTORY,
  LINE,
  WHATSAPP,
  QQ,
  WECHAT,
  SMS,
  EMAIL,
  COPYURL,
  ETC
}

enum AdBrixSignUpChannel {
  Kakao,
  Naver,
  Line,
  Google,
  Facebook,
  Twitter,
  WhatsApp,
  QQ,
  WeChat,
  UserId,
  ETC,
  SkTid,
  AppleId
}

extension _AdBrixSignUpChannelExtension on AdBrixSignUpChannel {
  int get intVal {
    return this.index + 1;
  }
}

enum AdBrixInviteChannel {
  Kakao,
  Naver,
  Line,
  Google,
  Facebook,
  Twitter,
  WhatsApp,
  QQ,
  WeChat,
  ETC
}

extension _AdBrixInviteChannelExtension on AdBrixInviteChannel {
  int get intVal {
    return this.index + 1;
  }
}

enum AdBrixCurrency {
  KR_KRW,
  US_USD,
  JP_JPY,
  EU_EUR,
  UK_GBP,
  CH_CNY,
  TW_TWD,
  HK_HKD,
  ID_IDR, //Indonesia
  IN_INR, //India
  RU_RUB, //Russia
  TH_THB, //Thailand
  VN_VND, //Vietnam
  MY_MYR //Malaysia
}

enum AdBrixEventUploadCountInterval { MIN, NORMAL, MAX }

enum AdBrixEventUploadTimeInterval { MIN, NORMAL, MAX }

enum AdBrixGender { FEMALE, MALE, UNKNOWN }

enum AdBrixPaymentMethod { CreditCard, BankTransfer, MobilePayment, ETC }

enum _AbxAPI {
  setEventUploadCountInterval,
  setEventUploadTimeInterval,
  login,
  logout,
  setAge,
  setGender,
  setUserProperties,
  commonSignUp,
  appUpdate,
  userInvite,
  useCredit,
  commonPurchase,
  event,
  commerceViewHome,
  commerceCategoryView,
  commerceProductView,
  commerceAddToCart,
  commerceAddToWishList,
  commerceReviewOrder,
  commerceRefund,
  commerceSearch,
  commerceShare,
  commerceListView,
  commerceCartView,
  commercePaymentInfoAdd,
  gameTutorialComplete,
  gameCharacterCreated,
  gameStageCleared,
  gameLevelAchieved,
  gdprForgetMe,
  setRegistrationId,
  startGettingIDFA,
  stopGettingIDFA,
  setPushEnable,
  fetchInAppMessage,
  getAllInAppMessage,
  openInAppMessage,
  setNotificationChannel,
  setSubscriptionStatus,
  getSubscriptionStatus,
  setKakaoId,
  setPhoneNumber
}
